from .functions import barplot, lineplot, scatterplot, jointplot, swarmplot
from .colormap import plot_colormap_gradient, plot_colormap_in_rgb_space, create_custom_colormap, delta_e, delta_e_lightness, convert_colormap_for_colorblindness, compare_colormaps

__version__ = '0.2.2'
__author__ = 'Molly Nelson'

print("Package Landborn 0.2.2 Successfully Imported")